@@ -15,43 +15,26 @@ sub demand
 {
 	my ($self, $def, $storage, $obj, $member, $class) = @_;
 
-	my (@coll, @lost);
+	print $Tangram::TRACE "loading $member\n" if $Tangram::TRACE;
+   
+	my @coll;
 
-	if (my $prefetch = $storage->{PREFETCH}{$class}{$member}{$storage->export_object($obj)})
+	if (my $prefetch = $storage->{PREFETCH}{$class}{$member}{$storage->id($obj)})
 	{
-	    print $Tangram::TRACE "demanding ".$storage->id($obj)
-		.".$member from prefetch\n" if $Tangram::TRACE;
 		@coll = @$prefetch;
 	}
 	else
 	{
-	    print $Tangram::TRACE "demanding ".$storage->id($obj)
-		.".$member from storage\n" if $Tangram::TRACE;
 		my $cursor = $self->cursor($def, $storage, $obj, $member);
 
-		for (my $item = $cursor->select(); $item; $item = $cursor->next)
+		for (my $item = $cursor->select; $item; $item = $cursor->next)
 		{
 			my $slot = shift @{ $cursor->{-residue} };
-			if (defined $slot) {
-                           $coll[$slot] = $item;
-                       } else {
-                           warn "object ".$storage->id($item)." has no slot in array ".$storage->id($obj)."/$member!";
-			   push @lost, $item
-                       }
-		}
-		# last-ditch effort to automatically DTRT
-		if (@lost) {
-		    foreach(@coll) {
-			if (!defined $_) {
-			    $_ = shift @lost;
-			}
-			last unless @lost;
-		    }
-		    push @coll, @lost;
+			$coll[$slot] = $item;
 		}
 	}
 
-	$self->set_load_state($storage, $obj, $member, [ map { ($_) ? $storage->id($_) : (die "No ID for $_") } @coll ]);
+	$self->set_load_state($storage, $obj, $member, [ map { $_ && $storage->id($_) } @coll ]);
 
 	return \@coll;
 }
