@@ -15,29 +15,17 @@ use Tangram::Relational;
 
 package Tangram;
 
-use vars qw( $TRACE $DEBUG_LEVEL );
+use vars qw( $TRACE );
 $TRACE = (\*STDOUT, \*STDERR)[$ENV{TANGRAM_TRACE} - 1] || \*STDERR
   if exists $ENV{TANGRAM_TRACE} && $ENV{TANGRAM_TRACE};
 
-
-$DEBUG_LEVEL = 0;
-
-package Tangram::Core;
-
-use Exporter;
-use vars qw(@ISA @EXPORT_OK);
-
-BEGIN {
-    @ISA = qw(Exporter);
-    @EXPORT_OK = qw(pretty);
-}
-
 sub pretty {
     my $thingy = shift;
+
     if (defined($thingy)) {
 	return "`$thingy'";
     } else {
-	return "undef";
+	return undef;
     }
 }
 
