@@ -136,7 +136,15 @@ sub build_select
 
 	if (my $order = $self->{-order})
 	{
-		$select .= "\n\tORDER BY " . join ', ', map { $_->{expr} } @$order;
+           # add the fields that we're ordering by to the select part
+           # of the query
+           my @not_in_it = (grep { $select !~ m/ \Q$_\E(?:,|$)/ }
+			    map { $_->{expr} } @$order);
+           $select =~ s{\n}{join("", map {", $_"} @not_in_it)."\n"}se
+	       if @not_in_it;
+
+           $select .= ("\n\tORDER BY ".
+		       join ', ', map { $_->{expr} } @$order);
 	}
 
 	if ($self->{-desc})
