@@ -5,7 +5,8 @@ use strict;
 package Tangram::IntrArray;
 
 use Tangram::AbstractArray;
-use base qw( Tangram::AbstractArray );
+use vars qw(@ISA);
+ @ISA = qw( Tangram::AbstractArray );
 
 use Carp;
 
@@ -23,8 +24,10 @@ sub reschema
 			$members->{$member} = $def;
 		}
 
-		$def->{coll} ||= $class . "_$member";
-		$def->{slot} ||= $class . "_$member" . "_slot";
+		$def->{coll} ||= ($schema->{normalize}->($class)
+				  . "_$member");
+		$def->{slot} ||= ($schema->{normalize}->($class)
+				  . "_$member" . "_slot");
    
 		$schema->{classes}{$def->{class}}{stateless} = 0;
 
