@@ -60,13 +60,13 @@ sub defered_save
 	$self->update($storage, $obj, $field,
 				  sub
 				  {
-					my $sql = "UPDATE $table SET $item_col = $coll_id WHERE $storage->{schema}{sql}{id_col} = @_";
+					my $sql = "UPDATE $table SET $item_col = $coll_id WHERE id = @_";
 					$storage->sql_do($sql);
 				  },
 				  
 				  sub
 				  {
-					my $sql = "UPDATE $table SET $item_col = NULL WHERE $storage->{schema}{sql}{id_col} = @_ AND $item_col = $coll_id";
+					my $sql = "UPDATE $table SET $item_col = NULL WHERE id = @_ AND $item_col = $coll_id";
 					$storage->sql_do($sql);
 				  } );
   }
@@ -77,16 +77,13 @@ sub demand
 
 	my $set = Set::Object->new();
 
-	if (my $prefetch = $storage->{PREFETCH}{$class}{$member}{$storage->export_object($obj)})
+	if (my $prefetch = $storage->{PREFETCH}{$class}{$member}{$storage->id($obj)})
 	{
-	    print $Tangram::TRACE "demanding ".$storage->id($obj)
-		.".$member from prefetch\n" if $Tangram::TRACE;
 		$set->insert(@$prefetch);
 	}
 	else
 	{
-	    print $Tangram::TRACE "demanding ".$storage->id($obj)
-		.".$member from storage\n" if $Tangram::TRACE;
+		print $Tangram::TRACE "loading $member\n" if $Tangram::TRACE;
 
 		my $cursor = Tangram::CollCursor->new($storage, $def->{class}, $storage->{db});
 
@@ -118,7 +115,7 @@ sub erase
 		}
 		else
 		{
-			my $item_classdef = $storage->{schema}{classes}{$def->{class}};
+			my $item_classdef = $storage->{schema}{$def->{class}};
 			my $table = $item_classdef->{table} || $def->{class};
 			my $item_col = $def->{coll};
 			$storage->sql_do("UPDATE $table SET $item_col = NULL WHERE $item_col = $coll_id");
@@ -141,11 +138,18 @@ sub remote_expr
 sub prefetch
 {
 	my ($self, $storage, $def, $coll, $class, $member, $filter) = @_;
-
+   
 	my $ritem = $storage->remote($def->{class});
 
 	my $prefetch = $storage->{PREFETCH}{$class}{$member} ||= {}; # weakref
 
+	my $ids = $storage->my_select_data( cols => [ $coll->{id} ], filter => $filter );
+
+	while (my ($id) = $ids->fetchrow)
+	{
+		$prefetch->{$id} = []
+	}
+
 	my $includes = $coll->{$member}->includes($ritem);
 	$includes &= $filter if $filter;
 
@@ -154,7 +158,7 @@ sub prefetch
 	while (my $item = $cursor->current)
 	{
 		my ($coll_id) = $cursor->residue;
-		push @{ $prefetch->{$coll_id}||=[] }, $item;
+		push @{ $prefetch->{$coll_id} }, $item;
 		$cursor->next;
 	}
 
