@@ -14,21 +14,10 @@ sub FETCH
 {
    my $self = shift;
    my ($storage, $id, $member, $refid) = @$self;
-   my $refobj;
-
-   if ($id) {
-       print $Tangram::TRACE "demanding $id.$member\n"
-	   if $Tangram::TRACE;
-       my $obj = $storage->{objects}{$id};
-       $refobj = $storage->load($refid);
-       untie $obj->{$member};
-       $obj->{$member} = $refobj;
-   } else {
-       print $Tangram::TRACE "demanding obj $refid\n"
-	   if $Tangram::TRACE;
-       untie $$member;
-       $refobj = $$member = $storage->load($refid);
-   }
+   my $obj = $storage->{objects}{$id};
+   my $refobj = $storage->load($refid);
+   untie $obj->{$member};
+   $obj->{$member} = $refobj;
    return $refobj;
 }
 
@@ -36,20 +25,15 @@ sub STORE
 {
    my ($self, $val) = @_;
    my ($storage, $id, $member, $refid) = @$self;
-   if ($id) {
-       my $obj = $storage->{objects}{$id};
-       untie $obj->{$member};
-       return $obj->{$member} = $val;
-   } else {
-       untie $$member;
-       $$member = $val;
-   }
+   my $obj = $storage->{objects}{$id};
+   untie $obj->{$member};
+   return $obj->{$member} = $val;
 }
 
 sub id
 {
    my ($storage, $id, $member, $refid) = @{shift()};
-   $refid;
+   $refid
 }
 
 use Tangram::Scalar;
@@ -57,7 +41,7 @@ use Tangram::Scalar;
 package Tangram::Ref;
 
 use vars qw(@ISA);
- @ISA = qw( Tangram::Scalar );
+ @ISA = qw( Tangram::Scalar SelfLoader );
 
 $Tangram::Schema::TYPES{ref} = Tangram::Ref->new;
 
@@ -87,7 +71,7 @@ sub get_exporter
 
 	my $field = $self->{name};
 	my $table = $context->{class}{table};
-	my $deep_update = $self->{deep_update};
+	my $deep_update = exists $self->{deep_update};
 	
 	if ($context->{layout1}) {
 	  return sub {
@@ -96,7 +80,6 @@ sub get_exporter
 		return undef unless exists $obj->{$field};
 		
 		my $storage = $context->{storage};
-		my $schema = $storage->{schema};
 		
 		my $tied = tied($obj->{$field});
 		return $tied->id if $tied;
@@ -114,7 +97,7 @@ sub get_exporter
 							 # now that the object has been saved, we have an id for it
 							 my $refid = $storage->id($ref);
 							 # patch the column in the referant
-							 $storage->sql_do( "UPDATE $table SET $self->{col} = $refid WHERE $schema->{sql}{id_col} = $id" );
+							 $storage->sql_do( "UPDATE $table SET $self->{col} = $refid WHERE id = $id" );
 						   } );
 		  
 		  return undef;
@@ -147,14 +130,13 @@ sub get_exporter
 		$storage->defer( sub
 						 {
 						   my $storage = shift;
-						   my $schema = $storage->{schema};
-
+						   
 						   # now that the object has been saved, we have an id for it
 						   my $ref_id = $storage->export_object($ref);
 						   my $type_id = $storage->class_id(ref($ref));
 						   
 						   # patch the column in the referant
-						   $storage->sql_do( "UPDATE $table SET $self->{col} = $ref_id, $self->{type_col} = $type_id WHERE $schema->{sql}{id_col} = $exp_id" );
+						   $storage->sql_do( "UPDATE $table SET $self->{col} = $ref_id, $self->{type_col} = $type_id WHERE id = $exp_id" );
 						 } );
 		
 		return (undef, undef);
@@ -227,7 +209,15 @@ sub erase
 	}
 }
 
-sub coldefs
+sub DESTROY { }
+
+use SelfLoader;
+
+1;
+
+__DATA__
+
+sub Tangram::Ref::coldefs
 {
     my ($self, $cols, $members, $schema) = @_;
 
@@ -237,8 +227,3 @@ sub coldefs
 	  $cols->{ $def->{type_col} or die } = $schema->{sql}{cid} . $nullable;
     }
 }
-
-sub DESTROY { }
-
-1;
-
